///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: COMNotify.cpp $
// Version        : $Revision: 1 $
// Function       :
//
// Author         : $Author: Len $
// Date           : $Date: 30/05/02 19:20 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /Web Articles/COMNotify/COMNotify/COMNotify.cpp $
// 
// 1     30/05/02 19:20 Len
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All 
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors 
// shall not be liable for any damages suffered by licensee as a result of 
// using, modifying or distributing the software or its derivatives. In no
// event will JetByte Limited be liable for any lost revenue, profit or data,
// or for direct, indirect, special, consequential, incidental or punitive
// damages, however caused and regardless of the theory of liability, arising 
// out of the use of or inability to use software, even if JetByte Limited 
// has been advised of the possibility of such damages.
//
// This software is not designed or intended for use in on-line control of 
// aircraft, air traffic, aircraft navigation or aircraft communications; or in 
// the design, construction, operation or maintenance of any nuclear 
// facility. Licensee represents and warrants that it will not use or 
// redistribute the Software for such purposes. 
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools\NMTools\NMConferenceManager.h"
#include "JetByteTools\Win32Tools\Utils.h"
#include "JetByteTools\Win32Tools\AutoResetEvent.h"
#include "JetByteTools\COMTools\Utils.h"

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::NetMeeting::CNmException;
using JetByteTools::NetMeeting::CNmConferenceManager;
using JetByteTools::Win32::Output;
using JetByteTools::Win32::CAutoResetEvent;
using JetByteTools::COM::WaitWithMessageLoop;

///////////////////////////////////////////////////////////////////////////////
// Function prototypes
///////////////////////////////////////////////////////////////////////////////

BOOL __stdcall CtrlHandler(DWORD fdwCtrlType);

static LPCTSTR DecodeNMUIEvent(
   CONFN uNotify);

///////////////////////////////////////////////////////////////////////////////
// MyConferenceManager
///////////////////////////////////////////////////////////////////////////////

class MyConferenceManager : public CNmConferenceManager
{
   public :
   
		HRESULT STDMETHODCALLTYPE NmUI(CONFN confn)
      {
         Output(DecodeNMUIEvent(confn));
	      return S_OK;
      }

		HRESULT STDMETHODCALLTYPE ConferenceCreated(INmConference *pConference)
      {
         Output(_T("MyConferenceManager::CallCreated"));
	      return S_OK;
      }

      HRESULT STDMETHODCALLTYPE CallCreated(INmCall *pCall)
      {
         Output(_T("MyConferenceManager::ConferenceCreated"));
	      return S_OK;
      }
};

///////////////////////////////////////////////////////////////////////////////
// Program entry point
///////////////////////////////////////////////////////////////////////////////

int main(int argc, char* argv[])
{
   try
   {
      if (!SetConsoleCtrlHandler(CtrlHandler,TRUE))
      {
         MessageBox(0, _T("Could not set console handler"), _T("COMNotify"), MB_ICONSTOP);
         return 0;
      }

      if (MyConferenceManager::IsNmInstalled())
      {
         MyConferenceManager confManager;

		   ULONG options = NM_INIT_NORMAL;
		   ULONG capabilities = NMCH_ALL;

         confManager.Initialize(options, capabilities);

         CAutoResetEvent stopEvent(_T("JetByteToolsCOMNotifyShutdown"), false);

         WaitWithMessageLoop(stopEvent.GetEvent());

         Output(_T("Done"));
      }
      else
      {
         ::MessageBox(0, _T("This sample requires that Microsoft NetMeeting is installed."), _T("COMNotify"), MB_ICONSTOP);
      }
   }
   catch(CNmException &e)
   {
      e.MessageBox(0);
   }
   catch(...)
   {
      Output(_T("Unexpected exception"));
   }
   return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Helper functions
///////////////////////////////////////////////////////////////////////////////

BOOL __stdcall CtrlHandler(DWORD fdwCtrlType)
{ 
   CAutoResetEvent stopEvent(_T("JetByteToolsCOMNotifyShutdown"), false);
   
   stopEvent.Set();

   return TRUE;
}

static LPCTSTR DecodeNMUIEvent(
   CONFN uNotify)
{
	LPCTSTR pWhat = _T("NmUI - Unknown event");

	switch (uNotify)
	{
		case CONFN_FT_UI : 
			pWhat = _T("FT UI");
		break;

		case CONFN_FT_OFFERED : 
			pWhat = _T("FT OFFERED");
		break;

		case CONFN_FT_STARTED : 
			pWhat = _T("FT STARTED");
		break;

		case CONFN_FT_PROGRESS : 
			pWhat = _T("FT PROGRESS");
		break;

		case CONFN_FT_COMPLETE : 
			pWhat = _T("FT COMPLETE");
		break;

		case CONFN_FT_CANCELED : 
			pWhat = _T("FT CANCELED");
		break;

		case CONFN_CLICK_CONTROL : 
			pWhat = _T("CLICK CONTROL");
		break;

		case CONFN_CLICK_OBSCURE : 
			pWhat = _T("CLICK OBSCURE");
		break;

		case CONFN_CLICK_REMOTE_NOT_COLLABORATING : 
			pWhat = _T("CLICK REMOTE NOT COLLABORATING");
		break;

		case CONFN_CLICK_LOCAL_NOT_COLLABORATING : 
			pWhat = _T("CLICK LOCAL NOT COLLABORATING");
		break;

		case CONFN_CALL_INCOMPATIBLE : 
			pWhat = _T("CALL INCOMPATIBLE");
		break;

		case CONFN_CALL_OLDER : 
			pWhat = _T("CALL OLDER");
		break;

		case CONFN_CALL_NEWER : 
			pWhat = _T("CALL NEWER");
		break;

		case CONFN_CALL_IGNORED : 
			pWhat = _T("CALL IGNORED");
		break;

		case CONFN_CALL_FAILED : 
			pWhat = _T("CALL FAILED");
		break;

		case CONFN_CALL_IN_CONFERENCE : 
			pWhat = _T("CALL IN CONFERENCE");
		break;

		case CONFN_NM_STARTED : 
			pWhat = _T("NM STARTED");
		break;

		case CONFN_NM_STOPPED : 
			pWhat = _T("NM STOPPED");
		break;
	}

   return pWhat;
}

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
