///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: Notify.cpp $
// Version        : $Revision: 3 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 30/05/02 15:45 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/COMTools/Notify.cpp $
// 
// 3     30/05/02 15:45 Len
// Lint issues.
// 
// 2     30/05/02 11:03 Len
// Code clean
// 
// 1     22/05/02 11:05 Len
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1997 - 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All 
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors 
// shall not be liable for any damages suffered by licensee as a result of 
// using, modifying or distributing the software or its derivatives. In no
// event will JetByte Limited be liable for any lost revenue, profit or data,
// or for direct, indirect, special, consequential, incidental or punitive
// damages, however caused and regardless of the theory of liability, arising 
// out of the use of or inability to use software, even if JetByte Limited 
// has been advised of the possibility of such damages.
//
// This software is not designed or intended for use in on-line control of 
// aircraft, air traffic, aircraft navigation or aircraft communications; or in 
// the design, construction, operation or maintenance of any nuclear 
// facility. Licensee represents and warrants that it will not use or 
// redistribute the Software for such purposes. 
//
///////////////////////////////////////////////////////////////////////////////

#include "Notify.h"

#include "Exception.h"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Lint options
//
//lint -save
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools::COM
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {
namespace COM {

///////////////////////////////////////////////////////////////////////////////
// CNotify
///////////////////////////////////////////////////////////////////////////////

CNotify::CNotify()
	:	m_dwCookie(0),
		m_pIUnknown(0),
		m_pIConnectionPoint(0),
		m_pIConnectionPointContainer(0)
{

}

CNotify::~CNotify()
{
   try
   {
	   Disconnect();
   }
   catch(...)
   {
   }

   m_pIConnectionPoint = 0;
   m_pIConnectionPointContainer = 0;
   m_pIUnknown = 0;
}

void CNotify::Connect(
	IUnknown *pConnectTo, 
	REFIID riid, 
	IUnknown *pIUnknown)
{
	Disconnect();		// Tear down any existing connection...

	HRESULT hr;

	// Get the connection container

	hr = pConnectTo->QueryInterface(
		IID_IConnectionPointContainer, 
		(void **)&m_pIConnectionPointContainer);

	Exception::ThrowOnFailure(_T("CNotify::Connect() - QueryInterface"), hr);

	// Find an appropriate connection point

	hr = m_pIConnectionPointContainer->FindConnectionPoint(
		riid, 
		&m_pIConnectionPoint);

	Exception::ThrowOnFailure(_T("CNotify::Connect() - QueryInterface"), hr);

	DWORD cookie;

	// Connect the sink object
	hr = m_pIConnectionPoint->Advise((IUnknown*)pIUnknown, &cookie);

	Exception::ThrowOnFailure(_T("CNotify::Connect() - Advise"), hr);

	m_dwCookie = cookie;
}

void CNotify::Disconnect()
{
	if (0 != m_dwCookie)
	{
		// Disconnect the sink object
		
      //lint -e{613} Possible use of null pointer
      //lint -e{534} Ignoring return value of function
      m_pIConnectionPoint->Unadvise(m_dwCookie);
		m_dwCookie = 0;

      //lint -e{613} Possible use of null pointer
      //lint -e{534} Ignoring return value of function
		m_pIConnectionPoint->Release();
		m_pIConnectionPoint = 0;

      //lint -e{613} Possible use of null pointer
      //lint -e{534} Ignoring return value of function
		m_pIConnectionPointContainer->Release();
		m_pIConnectionPointContainer = 0;

		m_pIUnknown = 0;
	}
}

bool CNotify::Connected() const
{
	return (m_dwCookie != 0);
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools::COM
///////////////////////////////////////////////////////////////////////////////

} // End of namespace COM
} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// Lint options
//
//lint -restore
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
