///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: NMConferenceManager.cpp $
// Version        : $Revision: 2 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 30/05/02 18:19 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/NMTools/NMConferenceManager.cpp $
// 
// 2     30/05/02 18:19 Len
// Lint issues.
// 
// 1     22/05/02 15:15 Len
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1997 - 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All 
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors 
// shall not be liable for any damages suffered by licensee as a result of 
// using, modifying or distributing the software or its derivatives. In no
// event will JetByte Limited be liable for any lost revenue, profit or data,
// or for direct, indirect, special, consequential, incidental or punitive
// damages, however caused and regardless of the theory of liability, arising 
// out of the use of or inability to use software, even if JetByte Limited 
// has been advised of the possibility of such damages.
//
// This software is not designed or intended for use in on-line control of 
// aircraft, air traffic, aircraft navigation or aircraft communications; or in 
// the design, construction, operation or maintenance of any nuclear 
// facility. Licensee represents and warrants that it will not use or 
// redistribute the Software for such purposes. 
//
///////////////////////////////////////////////////////////////////////////////

#include "NmConferenceManager.h"
#include "JetByteTools\COMTools\Exception.h"
#include "JetByteTools\Win32Tools\OleString.h"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Lint options
//
//lint -save
//
//lint -esym(1928, TNotify) didn't appear in ctor initialiser list
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Win32::COleString;
using JetByteTools::COM::TNotify;

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools::NetMeeting
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {
namespace NetMeeting {

///////////////////////////////////////////////////////////////////////////////
// CNmConferenceManager
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Static members
///////////////////////////////////////////////////////////////////////////////

bool CNmConferenceManager::IsNmInstalled()
{
	return E_FAIL != CheckInstalled(); 
}

HRESULT CNmConferenceManager::CheckInstalled()
{
	HRESULT hrInstalled = E_FAIL;			// No Netmeeting components are installed

	INmManager *pIManager = 0;

   JetByteTools::COM::CUsesCOM usesCOM(COINIT_APARTMENTTHREADED);

	HRESULT hr = CoCreateInstance(
		CLSID_NmManager,
		0,
		CLSCTX_INPROC,
		IID_INmManager,
		(void**)&pIManager);

	if (SUCCEEDED(hr))
	{
		INmSysInfo *pISysInfo = 0;

		hr = pIManager->GetSysInfo(&pISysInfo);
		
		if (SUCCEEDED(hr))
		{
			hrInstalled = pISysInfo->IsInstalled();

			// E_FAIL -	Nothing installed (so how did we get this far?!)
			//
			// NM_E_NOT_INITIALIZED - One or more NetMeeting settings are not valid.  
			//
			// S_FALSE - The NetMeeting settings appear to be valid, but the 
			//           NetMeeting wizard has not run.  
			//
			// S_OK - Everything is installed properly 

         //lint -e{534} Ignoring return value of function
			pISysInfo->Release();
		}

      //lint -e{534} Ignoring return value of function
		pIManager->Release();
	}

	return hrInstalled;
}

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

CNmConferenceManager::CNmConferenceManager()
	:	CUsesCOM(COINIT_APARTMENTTHREADED), // Doesnt work if we're in the MTA
		m_pIManager(0)
{
	HRESULT hr = CoCreateInstance(
		CLSID_NmManager,
		0,
		CLSCTX_INPROC,
		IID_INmManager,
		(void**)&m_pIManager);
 
	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::CNmConferenceManager() - CoCreateInstance()"), hr);
	}

	// Connect to the conference manager object

   TNotify<INmManagerNotify, &IID_INmManagerNotify>::Connect(m_pIManager);
}

CNmConferenceManager::~CNmConferenceManager()
{
   try
   {
   	if (m_pIManager)
	   {
         //lint -e{534} Ignoring return value of function
   		m_pIManager->Release();
	   	m_pIManager = 0;
	   }
   }
   catch(...)
   {
   }
}

///////////////////////////////////////////////////////////////////////////////
// Implement INmManager
///////////////////////////////////////////////////////////////////////////////

void CNmConferenceManager::Initialize(
	ULONG &options, 
	ULONG &uChannelCaps)
{	
	HRESULT hr = m_pIManager->Initialize(&options, &uChannelCaps);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::Initialize()"), hr);
	}
}

INmConference *CNmConferenceManager::CreateConference(
	LPCTSTR pName, 
	//LPCTSTR pPassword,
	ULONG channelCapabilities)
{
	INmConference *pConference = 0;

	BSTR name = SysAllocString(COleString(pName));

	//BSTR password = SysAllocString(COleString(pPassword));

	HRESULT hr = m_pIManager->CreateConference(
		&pConference,
		name,
		0,//password,
		channelCapabilities);

	SysFreeString(name);
	//SysFreeString(password);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::CreateConference()"), hr);
	}

	return pConference;
}

INmCall *CNmConferenceManager::CallConference(
	NM_CALL_TYPE callType,
	NM_ADDR_TYPE addressType,
	LPCTSTR pAddress, 
	LPCTSTR pName, 
	LPCTSTR pPassword /* = _T("") */)
{
	INmCall *pCall = 0;

	BSTR addr = SysAllocString(COleString(pAddress));
	BSTR name = SysAllocString(COleString(pName));
	BSTR password = SysAllocString(COleString(pPassword));

	HRESULT hr = m_pIManager->CallConference(
		&pCall,
		callType,
		addressType,
		addr,
		name,
		0);		// Password currently not implemented

	SysFreeString(addr);
	SysFreeString(name);
	SysFreeString(password);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::CallConference()"), hr);
	}

	return pCall;
}

INmSysInfo *CNmConferenceManager::GetSysInfo()
{
	INmSysInfo *pSysInfo = 0;

	HRESULT hr = m_pIManager->GetSysInfo(&pSysInfo);

	if (FAILED(hr))
	{
		throw Exception(_T("CNmConferenceManager::GetSysInfo()"), hr);
	}

	return pSysInfo;
}

CNmConferenceManager::operator INmManager *()
{
	return m_pIManager;
}

///////////////////////////////////////////////////////////////////////////////
// Implement INmConferenceNotify
///////////////////////////////////////////////////////////////////////////////

HRESULT STDMETHODCALLTYPE CNmConferenceManager::NmUI(
	CONFN /* confn */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmConferenceManager::CallCreated(
	INmCall * /* pCall */)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CNmConferenceManager::ConferenceCreated(
	INmConference * /* pConference */)
{
	return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools::NetMeeting
///////////////////////////////////////////////////////////////////////////////

} // End of namespace NetMeeting
} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// Lint options
//
//lint -restore
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////
