///////////////////////////////////////////////////////////////////////////////
//
// File           : $Workfile: Olestring.cpp $
// Version        : $Revision: 1 $
// Function       : 
//
// Author         : $Author: Len $
// Date           : $Date: 22/05/02 9:12 $
//
// Notes          : 
//
// Modifications  :
//
// $Log: /JetByteTools/Win32Tools/Olestring.cpp $
// 
// 1     22/05/02 9:12 Len
// 
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 1997 - 2002 JetByte Limited.
//
// JetByte Limited grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All 
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors 
// shall not be liable for any damages suffered by licensee as a result of 
// using, modifying or distributing the software or its derivatives. In no
// event will JetByte Limited be liable for any lost revenue, profit or data,
// or for direct, indirect, special, consequential, incidental or punitive
// damages, however caused and regardless of the theory of liability, arising 
// out of the use of or inability to use software, even if JetByte Limited 
// has been advised of the possibility of such damages.
//
// This software is not designed or intended for use in on-line control of 
// aircraft, air traffic, aircraft navigation or aircraft communications; or in 
// the design, construction, operation or maintenance of any nuclear 
// facility. Licensee represents and warrants that it will not use or 
// redistribute the Software for such purposes. 
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Lint options
//
//lint -save
//
///////////////////////////////////////////////////////////////////////////////

#include "OleString.h"

#pragma warning(disable: 4711) // function selected for automatic inline expansion

///////////////////////////////////////////////////////////////////////////////
// Lint options
//
//lint -save
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools::Win32
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools {
namespace Win32 {

///////////////////////////////////////////////////////////////////////////////
// Static functions defined in this file
///////////////////////////////////////////////////////////////////////////////

static char *ustrdup(const wchar_t *pIn);
static wchar_t *ustrdup(const char *pIn);

///////////////////////////////////////////////////////////////////////////////
// COleString
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Construction and destruction
///////////////////////////////////////////////////////////////////////////////

COleString::COleString(LPCOLESTR lpOleString)
   : m_bOleString(true), m_lpOleString((LPOLESTR)lpOleString), m_lpString(0)
{
}

COleString::COleString(LPCSTR lpString)
   : m_bOleString(false), m_lpOleString(0), m_lpString((LPSTR)lpString)
{
}

COleString::~COleString()
{
   if (m_bOleString)
   {
      // Free the char * string rep as we own it.
   
      free(m_lpString);

      m_lpOleString = 0;
   }
   else
   {
      free(m_lpOleString);
      
      m_lpString = 0;
   }
}

///////////////////////////////////////////////////////////////////////////////
// Conversion 
///////////////////////////////////////////////////////////////////////////////

COleString::operator const char *() const
{
   return GetAs_char();
}

COleString::operator const wchar_t *() const
{
	return GetAs_wchar_t();
}

const char *COleString::GetAs_char() const
{
   if (!m_lpString)
   {
      m_lpString = ustrdup(m_lpOleString);
   }

   return m_lpString;
}

const wchar_t *COleString::GetAs_wchar_t() const
{
   if (!m_lpOleString)
   {
      m_lpOleString = ustrdup(m_lpString);
   }

   return m_lpOleString;
}

const TCHAR *COleString::GetAs_tchar() const
{
#ifdef  _UNICODE
   return GetAs_wchar_t();
#else
   return GetAs_char();
#endif // _UNICODE
}

///////////////////////////////////////////////////////////////////////////////
// Duplication
///////////////////////////////////////////////////////////////////////////////

char *COleString::DuplicateAs_char() const
{
   return strdup(GetAs_char());
}

wchar_t *COleString::DuplicateAs_wchar_t() const
{
   return wcsdup(GetAs_wchar_t());
}

TCHAR *COleString::DuplicateAs_tchar() const
{
#ifdef  _UNICODE
   return DuplicateAs_wchar_t();
#else
   return DuplicateAs_char();
#endif // _UNICODE
}

///////////////////////////////////////////////////////////////////////////////
// Static functions
///////////////////////////////////////////////////////////////////////////////

wchar_t *ustrdup(const char *pIn)
{
   size_t length = strlen(pIn) + 1;
    
   wchar_t *pOut = (wchar_t*)malloc((sizeof(wchar_t) * length));
    
   if (pOut)
   {
      mbstowcs(pOut, pIn, length);
   }

   return pOut;
}

char *ustrdup(const wchar_t *pIn)
{
   size_t length = wcslen(pIn) + 1;
    
   char *pOut = (char*)malloc((sizeof(char) * length));
    
   if (pOut)
   {
      wcstombs(pOut, pIn, length);
   }

   return pOut;
}

///////////////////////////////////////////////////////////////////////////////
// Namespace: JetByteTools::Win32
///////////////////////////////////////////////////////////////////////////////

} // End of namespace Win32
} // End of namespace JetByteTools 

///////////////////////////////////////////////////////////////////////////////
// Lint options
//
//lint -restore
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// End of file...
///////////////////////////////////////////////////////////////////////////////

